"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@typescript-eslint/utils");
const columnType_1 = require("../utils/columnType");
const treeTraversal_1 = require("../utils/treeTraversal");
const createRule = utils_1.ESLintUtils.RuleCreator((name) => `https://github.com/daniel7grant/eslint-plugin-typeorm-typescript#typeorm-typescript${name}`);
const enforceColumnTypes = createRule({
    name: 'enforce-column-types',
    defaultOptions: [],
    meta: {
        type: 'problem',
        docs: {
            description: 'TypeORM and TypeScript types should be the same on columns.',
            recommended: 'recommended',
        },
        hasSuggestions: true,
        messages: {
            typescript_typeorm_column_mismatch: 'Type of {{ propertyName }}{{ className }} is not matching the TypeORM column type{{ expectedValue }}.',
            typescript_typeorm_column_suggestion: 'Change the type of {{ propertyName }} to {{ expectedValue }}.',
        },
        schema: [],
    },
    create(context) {
        return {
            PropertyDefinition(node) {
                var _a;
                const columnArguments = (0, treeTraversal_1.findDecoratorArguments)(node.decorators, 'Column');
                if (!(columnArguments === null || columnArguments === void 0 ? void 0 : columnArguments[0]) || !node.typeAnnotation) {
                    return;
                }
                const typeormType = (0, columnType_1.convertArgumentToColumnType)(columnArguments[0]);
                const { typeAnnotation } = node.typeAnnotation;
                const typescriptType = (0, columnType_1.convertTypeToColumnType)(typeAnnotation);
                if (!(0, columnType_1.isTypesEqual)(typeormType, typescriptType)) {
                    const fixReplace = (0, columnType_1.typeToString)(typeormType, {
                        literal: typescriptType === null || typescriptType === void 0 ? void 0 : typescriptType.literal,
                    });
                    // Construct strings for error message
                    const propertyName = ((_a = node.key) === null || _a === void 0 ? void 0 : _a.type) === utils_1.AST_NODE_TYPES.Identifier ? node.key.name : 'property';
                    const classObject = (0, treeTraversal_1.findParentClass)(node);
                    const className = (classObject === null || classObject === void 0 ? void 0 : classObject.id) ? ` in ${classObject.id.name}` : '';
                    const expectedValue = fixReplace ? ` (expected type: ${fixReplace})` : '';
                    // Report the error
                    context.report({
                        node,
                        messageId: 'typescript_typeorm_column_mismatch',
                        data: {
                            className,
                            propertyName,
                            expectedValue,
                        },
                        suggest: fixReplace
                            ? [
                                {
                                    messageId: 'typescript_typeorm_column_suggestion',
                                    fix: (fixer) => fixer.replaceText(typeAnnotation, fixReplace),
                                    data: {
                                        propertyName,
                                        expectedValue: fixReplace,
                                    },
                                },
                            ]
                            : undefined,
                        loc: node.loc,
                    });
                }
            },
        };
    },
});
exports.default = enforceColumnTypes;
