"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@typescript-eslint/utils");
const treeTraversal_1 = require("../utils/treeTraversal");
const relationType_1 = require("../utils/relationType");
const createRule = utils_1.ESLintUtils.RuleCreator((name) => `https://github.com/daniel7grant/eslint-plugin-typeorm-typescript#typeorm-typescript${name}`);
const enforceColumnTypes = createRule({
    name: 'enforce-relation-types',
    defaultOptions: [],
    meta: {
        type: 'problem',
        docs: {
            description: 'TypeScript types should be consistent with the relations.',
            recommended: 'recommended',
        },
        hasSuggestions: true,
        messages: {
            typescript_typeorm_relation_mismatch: 'Type of {{ propertyName }}{{ className }} is not consistent with the TypeORM relation type {{ relation }}{{ expectedValue }}.',
            typescript_typeorm_relation_array_to_many: '{{ relation }} relations return multiple entities. Type of {{ propertyName }}{{ className }} should be an array {{ expectedValue }}.',
            typescript_typeorm_relation_nullable_by_default: 'TypeORM relations are nullable by default. Type of {{ propertyName }}{{ className }} should be nullable{{ expectedValue }}.',
            typescript_typeorm_relation_suggestion: 'Change the type of {{ propertyName }} to {{ expectedValue }}.',
            typescript_typeorm_relation_nullable_by_default_suggestion: 'Make the {{ relation }} relation nullable: false.',
        },
        schema: [],
    },
    create(context) {
        return {
            PropertyDefinition(node) {
                var _a;
                const relationsArguments = relationType_1.relationTypes.map((relation) => [
                    relation,
                    (0, treeTraversal_1.findDecoratorArguments)(node.decorators, relation),
                ]);
                const relationArguments = relationsArguments.find(([, args]) => args);
                if (!relationArguments) {
                    return;
                }
                const [relation, relArguments] = relationArguments;
                const typeormType = (0, relationType_1.convertArgumentToRelationType)(relation, relArguments);
                if (!node.typeAnnotation) {
                    return;
                }
                const { typeAnnotation } = node.typeAnnotation;
                const typescriptType = (0, relationType_1.convertTypeToRelationType)(typeAnnotation);
                if (!(0, relationType_1.isTypesEqual)(typeormType, typescriptType)) {
                    let messageId = 'typescript_typeorm_relation_mismatch';
                    const suggestions = [];
                    const fixReplace = (0, relationType_1.typeToString)(typeormType);
                    // Construct strings for error message
                    const propertyName = ((_a = node.key) === null || _a === void 0 ? void 0 : _a.type) === utils_1.AST_NODE_TYPES.Identifier ? node.key.name : 'property';
                    const classObject = (0, treeTraversal_1.findParentClass)(node);
                    const className = (classObject === null || classObject === void 0 ? void 0 : classObject.id) ? ` in ${classObject.id.name}` : '';
                    const expectedValue = fixReplace ? ` (expected type: ${fixReplace})` : '';
                    if (fixReplace) {
                        suggestions.push({
                            messageId: 'typescript_typeorm_relation_suggestion',
                            fix: (fixer) => fixer.replaceText(typeAnnotation, fixReplace),
                            data: {
                                propertyName,
                                expectedValue: fixReplace,
                            },
                        });
                    }
                    // Arrays are required for ToMany types, help with a custom message
                    if ((0, relationType_1.isTypeMissingArray)(typeormType, typescriptType)) {
                        messageId = 'typescript_typeorm_relation_array_to_many';
                    }
                    // Relations are nullable by default which can be confusing, help with a custom message
                    if (relArguments && (0, relationType_1.isTypeMissingNullable)(typeormType, typescriptType)) {
                        messageId = 'typescript_typeorm_relation_nullable_by_default';
                        const options = (0, treeTraversal_1.findObjectArgument)(relArguments);
                        if (!options) {
                            const lastArgument = relArguments[relArguments.length - 1];
                            suggestions.push({
                                messageId: 'typescript_typeorm_relation_nullable_by_default_suggestion',
                                fix: (fixer) => fixer.insertTextAfter(lastArgument, ', { nullable: false }'),
                                data: {
                                    relation,
                                },
                            });
                        }
                    }
                    // Report the error
                    context.report({
                        node,
                        messageId,
                        data: {
                            className,
                            propertyName,
                            expectedValue,
                            relation,
                        },
                        suggest: suggestions,
                        loc: node.loc,
                    });
                }
            },
        };
    },
});
exports.default = enforceColumnTypes;
